/*
 * grader-io - gpio.h
 *
 * Copyright (C) 2016  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

#ifndef GPIO_H_INCLUDED
#define GPIO_H_INCLUDED

#ifdef HAVE_NXP_GPIO
#include <compiler.h>
#include <asm_io.h>
#include <lpc176x.h>

#define NUM_GPIO	160

static inline u32 __gpio_read(unsigned int bank, unsigned int reg)
{
	return readl((u8 *) GPIO_BASE + (bank << 5) + reg);
}

static inline void __gpio_write(u32 x, unsigned int bank, unsigned int reg)
{
	writel(x, (u8 *) GPIO_BASE + (bank << 5) + reg);
}

#define gpio_get_value_const gpio_get_value_const
static inline int gpio_get_value_const(unsigned int gpio)
{
	unsigned int bank;
	unsigned int bit;
	u32 x;

	if (gpio >= NUM_GPIO)
		return -EINVAL;

	bank = gpio >> 5;
	bit = gpio & 0x1f;

	x = __gpio_read(bank, GPIO_FIOPIN);
	return (x & (1 << bit)) != 0;
}

#define gpio_set_value_const gpio_set_value_const
static inline void gpio_set_value_const(unsigned int gpio, int value)
{
	unsigned int bank;
	unsigned int bit;
	u32 x;

	if (unlikely(gpio >= NUM_GPIO))
		return;

	bank = gpio >> 5;
	bit = gpio & 0x1f;

	x = 1 << bit;
	if (value)
		__gpio_write(x, bank, GPIO_FIOSET);
	else
		__gpio_write(x, bank, GPIO_FIOCLR);
}
#endif

int gpio_direction_input(unsigned int gpio);
int gpio_direction_output(unsigned int gpio, int value);
void __gpio_set_value(unsigned int gpio, int value);
int __gpio_get_value(unsigned int gpio);

static inline int gpio_get_value(unsigned int gpio)
{
#ifdef gpio_get_value_const
	if (__builtin_constant_p(gpio))
		return gpio_get_value_const(gpio);
	else
#endif
		return __gpio_get_value(gpio);
}

static inline void gpio_set_value(unsigned int gpio, int value)
{
#ifdef gpio_set_value_const
	if (__builtin_constant_p(gpio))
		gpio_set_value_const(gpio, value);
	else
#endif
		__gpio_set_value(gpio, value);
}

#endif
