/*
 * grader-io - timer.h
 *
 * Copyright (C) 2017  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

#ifndef TIMER_H_INCLUDED
#define TIMER_H_INCLUDED

#ifdef HAVE_UNISTD_H
#define HZ	100
#else
#define HZ	10000
#endif

extern volatile unsigned long jiffies;

#ifdef HAVE_UNISTD_H
#define GETTIME_FREQUENCY	(100000000UL)
#else
#define GETTIME_FREQUENCY	(96000000UL)
#endif

#define TICKS_PER_US (GETTIME_FREQUENCY / 1000000UL)

#define time_after(a, b) ((long)((long)(b) - (long)(a)) < 0)
#define time_before(a, b) ((long)((long)(a) - (long)(b)) < 0)

static inline unsigned long us_to_ticks(unsigned long t)
{
	return t * TICKS_PER_US;
}

static inline unsigned long ticks_to_us(unsigned long t)
{
	return (t + TICKS_PER_US/2) / TICKS_PER_US;
}

unsigned int gettime(void);
void timer_init(void);

void schedule_timeout_uninterruptible(long timeout);
void sleep_to(unsigned long expire);

#endif
