/*
 * grader-rt - init.c
 *
 * Copyright (C) 2017  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <kernel.h>
#include <asm_io.h>
#include <lpc176x.h>

extern int main(char *argv[], int argc);

/*
 * Configure CPU clock, 96 MHz from 12 MHz external crystal,
 */
static void clock_init(void)
{
	u8 __iomem *syscon = ioremap(SYSCON_BASE, 1024);
	u32 x;
	
	x = readl(syscon + SYSCON_PCONP);
	x |= SYSCON_PCONP_PCUART2 | SYSCON_PCONP_PCUART3;
	x |= SYSCON_PCONP_PCQEI;
	writel(x, syscon + SYSCON_PCONP);

	/* Disconnect PLL */
	x = readl(syscon + SYSCON_PLL0STAT);
	while (x & SYSCON_PLL0STAT_CSTAT) {
		writel(SYSCON_PLL0CON_PLLE0, syscon + SYSCON_PLL0CON);
		writel(0xaa, syscon + SYSCON_PLL0FEED);
		writel(0x55, syscon + SYSCON_PLL0FEED);
		x = readl(syscon + SYSCON_PLL0STAT);
	}

	/* Disable PLL */
	do {
		writel(0, syscon + SYSCON_PLL0CON);
		writel(0xaa, syscon + SYSCON_PLL0FEED);
		writel(0x55, syscon + SYSCON_PLL0FEED);
		x = readl(syscon + SYSCON_PLL0STAT);
	} while (x & SYSCON_PLL0STAT_ESTAT);

	/* Without PLL we can clock CPU at full speed */
	writel(0, syscon + SYSCON_CCLKCFG);

	/* switch to Internal oscilator */
	writel(SYSCON_CLKSRCSEL_IRC, syscon + SYSCON_CLKSRCSEL);

	/* Enable Main oscilator */
	writel(SYSCON_SCS_OSCEN, syscon + SYSCON_SCS);
	do {
		cpu_relax();
		x = readl(syscon + SYSCON_SCS);
	} while (!(x & SYSCON_SCS_OSCSTAT));

	/* switch to Main oscilator */
	writel(SYSCON_CLKSRCSEL_MAIN, syscon + SYSCON_CLKSRCSEL);

	/*
	 * The main clock is 12 MHz. We want 96 MHz CPU speed.
	 * The minimal FCCO is is 288 MHz (3 * 96 MHz),
	 * 12 MHz * 2 * 12 = 288 MHz
	 *
	 * M = 12
	 * N = 1
	 */
	x = 11 << SYSCON_PLL0CFG_M_SHIFT;
	x |= 0 << SYSCON_PLL0CFG_N_SHIFT;
	writel(x, syscon + SYSCON_PLL0CFG);
	writel(0xaa, syscon + SYSCON_PLL0FEED);
	writel(0x55, syscon + SYSCON_PLL0FEED);

	/* Enable PLL */
	do {
		writel(SYSCON_PLL0CON_PLLE0, syscon + SYSCON_PLL0CON);
		writel(0xaa, syscon + SYSCON_PLL0FEED);
		writel(0x55, syscon + SYSCON_PLL0FEED);
		x = readl(syscon + SYSCON_PLL0STAT);
	} while (!(x & SYSCON_PLL0STAT_ESTAT));

	/*
	 * With PLL at 288 MHz, we need divider 3 for 96 MHz
	 */
	writel(2, syscon + SYSCON_CCLKCFG);

	readl(syscon + SYSCON_PCLKSEL0);
	x &= ~(SYSCON_PCLKSEL_MASK << SYSCON_PCLKSEL0_UART0_SHIFT);
	x &= ~(SYSCON_PCLKSEL_MASK << SYSCON_PCLKSEL0_UART1_SHIFT);
	x |= (SYSCON_PCLKSEL_DIV1 << SYSCON_PCLKSEL0_UART0_SHIFT);
	x |= (SYSCON_PCLKSEL_DIV1 << SYSCON_PCLKSEL0_UART1_SHIFT);
	writel(x, syscon + SYSCON_PCLKSEL0);

	readl(syscon + SYSCON_PCLKSEL1);
	x &= ~(SYSCON_PCLKSEL_MASK << SYSCON_PCLKSEL1_UART2_SHIFT);
	x &= ~(SYSCON_PCLKSEL_MASK << SYSCON_PCLKSEL1_UART3_SHIFT);
	x |= (SYSCON_PCLKSEL_DIV1 << SYSCON_PCLKSEL1_UART2_SHIFT);
	x |= (SYSCON_PCLKSEL_DIV1 << SYSCON_PCLKSEL1_UART3_SHIFT);
	writel(x, syscon + SYSCON_PCLKSEL1);

	/* At 96 MHz we need 5 clocks for accessing flash */
	x = readl(syscon + SYSCON_FLASHCFG);
	x &= ~SYSCON_FLASHCFG_AMASK;
	x |= SYSCON_FLASHCFG_A5;
	writel(x, syscon + SYSCON_FLASHCFG);

	do {
		cpu_relax();
		x = readl(syscon + SYSCON_PLL0STAT);
	} while (!(x & SYSCON_PLL0STAT_PLOCK0));

	do {
		writel(SYSCON_PLL0CON_PLLE0 | SYSCON_PLL0CON_PLLC0,
				syscon + SYSCON_PLL0CON);
		writel(0xaa, syscon + SYSCON_PLL0FEED);
		writel(0x55, syscon + SYSCON_PLL0FEED);
		x = readl(syscon + SYSCON_PLL0STAT);
	} while (!(x & SYSCON_PLL0STAT_CSTAT));

	iounmap(syscon);
}

#if 0
static const unsigned char pinmux_config[PIN_MAX + 1] = {
	[PIN_PIO0_8] = PINMUX_MODE_PULLUP | PINMUX_FUNC(0),	/* STATUS */
	[PIN_PIO1_9] = PINMUX_MODE_PULLUP | PINMUX_FUNC(0),	/* STATUS */
	[PIN_PIO1_6] = PINMUX_MODE_PULLUP | PINMUX_FUNC(1),	/* RXD */
	[PIN_PIO1_7] = PINMUX_MODE_PULLUP | PINMUX_FUNC(1),	/* TXD */
	[PIN_PIO1_0] = PINMUX_MODE_PULLUP | PINMUX_FUNC(1),	/* Out 0 */
	[PIN_PIO1_1] = PINMUX_MODE_PULLUP | PINMUX_FUNC(1),	/* Out 1 */
	[PIN_PIO1_2] = PINMUX_MODE_PULLUP | PINMUX_FUNC(1),	/* Out 2 */
	[PIN_PIO3_1] = PINMUX_MODE_PULLUP | PINMUX_FUNC(0),	/* Out 3 */
};

static void pinmux_init(void)
{
	u32 __iomem *iocon = ioremap(IOCON_BASE, 1024);
	unsigned int i;
	u32 x;

	for (i = 0; i < ARRAY_SIZE(pinmux_config); i++) {
		unsigned char config = pinmux_config[i];

		if (config & PINMUX_SET) {
			x = readl(iocon + i);
			x &= ~0x3f;
			x |= config & 0x3f;
			writel(x, iocon + i);
		}
	}
	iounmap(iocon);
}
#endif


#define PINMUX_BASE	0x4002c000
#define PINSEL0		0x000
#define PINSEL1		0x004
#define PINSEL3		0x00c
#define PINSEL9		0x024

static void pinmux_init(void)
{
	u8 __iomem *iocon = ioremap(PINMUX_BASE, 1024);

	writel(0x00500050, iocon + PINSEL0);
	writel(0x00004100, iocon + PINSEL3);
	writel(0x0f000000, iocon + PINSEL9);
//	writel(0x00000001, iocon + PINSEL1);
}

/*
 * copy .data section from flash to RAM
 */
static void copy_data(void)
{
	extern unsigned char __data_loadaddr;
	extern unsigned char __data_start;
	extern unsigned char __data_end;

	memcpy(&__data_start, &__data_loadaddr, &__data_end - &__data_start);
}

void start_kernel(void)
{
	static char *main_args[] = {
		NULL,
	};

	local_irq_disable();

	copy_data();
	clock_init();
	pinmux_init();
	timer_init();
	local_irq_enable();

	main(main_args, 0);
	for (;;);
}
