/*
 * grader-io - posix-timer.c
 *
 * Copyright (C) 2017  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <kernel.h>
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>
#include <signal.h>
#include <sys/time.h>
#include <timer.h>

volatile unsigned long jiffies = -300 * HZ;

void timer_irq_handler(void)
{
	jiffies++;
}

void sleep_to(unsigned long expire)
{
	unsigned long flags;

	local_irq_save(flags);
	while (time_before(jiffies, expire)) {
		local_irq_restore(flags);
		pause();
		local_irq_save(flags);
	}
	local_irq_restore(flags);
}

void schedule_timeout_uninterruptible(long timeout)
{
	sleep_to(jiffies + timeout);
}

void timer_init(void)
{
	const struct itimerval itv = {
		.it_interval = { .tv_sec = 0, .tv_usec = 1000 },
		.it_value = { .tv_sec = 0, .tv_usec = 1000 },
	};
	int ret;

	ret = setitimer(ITIMER_REAL, &itv, NULL);
	if (ret < 0) {
		perror("setitimer");
		exit(EXIT_FAILURE);
	}
}

unsigned int gettime(void)
{
	static unsigned long long prev_t;
	static unsigned long jiffies2;
	struct timespec ts;
	unsigned long long t;
	unsigned long long delta;
	int ret;

	ret = clock_gettime(CLOCK_MONOTONIC, &ts);
	if (ret < 0)
		return 0;
	t = (unsigned long long)(1000000000ULL * ts.tv_sec + ts.tv_nsec);
	delta = t - prev_t;
	delta /= 10;
	prev_t += 10 * delta;
	jiffies2 += delta;
	return (uint32_t) jiffies2;
}
